package mandelbrot;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.io.IOException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JPanel;

public class MandelbrotPanel extends JPanel {

    // Assign a string that contains the URL of
    // of an image on the Web in this constant.
    private static final String urlString
//            = "http://www.alachuacounty.us/Employment/"
//            + "Recruitment/living/PublishingImages/"
//            + "Poe_Springs_and_Santa_Fe_River.JPG";
            = "http://nssdc.gsfc.nasa.gov/"
            + "image/planetary/mars/marsglobe1.jpg";
    private static final Logger logger
            = Logger.getLogger(MandelbrotPanel.class.getName());

    
    private static final int BITMAP_WIDTH = 512;
    private static final int BITMAP_HEIGHT = 512;
    
    /**
     * By assigning different values to the following
     * constants, you can produce images of different
     * parts of the Mandelbrot set.
     */
    private static final double WORLD_XMIN = -1.0;
    private static final double WORLD_YMIN = -1.0;
    private static final double WORLD_WIDTH = 2.0;
    private static final double WORLD_HEIGHT = 2.0;
    private BufferedImage image;

    /**
     * Do not modify this constructor.
     */
    public MandelbrotPanel() {
        int w = BITMAP_WIDTH;
        int h = BITMAP_HEIGHT;
        int imageType = BufferedImage.TYPE_INT_RGB;
        this.image = new BufferedImage(w, h, imageType);
    } // MandelbrotPanel()

   /**
    * Modify this method only as indicated;
    * either call pixelPainting() or call imageProcessing().
    * 
    * @param g 
    */
    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2D = (Graphics2D) g;

        // Call either pixelPainting() or imageProcessing().
        pixelPainting( g2D );
//        imageProcessing( g2D );

    } // paintComponent( Graphics )

    /**
     * Do not modify this method.
     * 
     * @param g2D
     */
    private void pixelPainting( Graphics2D g2D ) {
        int w = this.getWidth();
        int h = this.getHeight();

        AffineTransform scale = new AffineTransform();
        double sw = ((double) w) / BITMAP_WIDTH;
        double sh = ((double) h) / BITMAP_HEIGHT;
        scale.setToScale(sw, sh);

        WritableRaster raster = this.image.getRaster();

        float[] colorComponents = new float[3];
        int[] rgb = new int[3];

        for (int i = 0; i < BITMAP_HEIGHT; i++) {
            double fractionOfHeight = ((double) i) / BITMAP_HEIGHT;
            double imag = WORLD_YMIN + fractionOfHeight * WORLD_HEIGHT;

            for (int j = 0; j < BITMAP_WIDTH; j++) {
                double fractionOfWidth = ((double) j) / BITMAP_WIDTH;
                double real = WORLD_XMIN + fractionOfWidth * WORLD_WIDTH;

                colorComponents = this.pixelColor(real, imag).getColorComponents(colorComponents);
                rgb[0] = (int) (255 * colorComponents[0]);
                rgb[1] = (int) (255 * colorComponents[1]);
                rgb[2] = (int) (255 * colorComponents[2]);
                raster.setPixel(i, j, rgb);
            } // for
        } // for

        g2D.drawImage(image, scale, this);
    } // pixelPainting( Graphics2D )
    
    /**
     * Modify the code within the nested loops to
     * produce other effects.
     * 
     * @param g2D 
     */
    private void imageProcessing( Graphics2D g2D ) {
        try {
            URL url = new URL(urlString);
            this.image = ImageIO.read(url);

            double panelWidth = this.getWidth();
            double panelHeight = this.getHeight();

            double imageWidth = image.getWidth(this);
            double imageHeight = image.getHeight(this);

            System.out.println("image width = " + imageWidth);
            System.out.println("image height = " + imageHeight);

            AffineTransform scaleImage = new AffineTransform();
            scaleImage.setToScale(panelWidth / imageWidth, panelHeight / imageHeight);

            int[] color = new int[3];
            WritableRaster raster = this.image.getRaster();

            System.out.println("raster width = " + raster.getWidth());
            System.out.println("raster height = " + raster.getHeight());

            for (int i = 0; i < raster.getHeight(); i++) {
                double t = ((double) i) / raster.getHeight();
                for (int j = 0; j < raster.getWidth(); j++) {
                    raster.getPixel(j, i, color);
                    
                    // Use weighted average of pixel's color
                    // and shade of gray to transition from full
                    // color at the top of the image to black and white
                    // at the bottom of the image.
                    int shade = (color[0] + color[1] + color[2]) / 3;
                    color[0] = (int) ((1 - t) * color[0] + t * shade);
                    color[1] = (int) ((1 - t) * color[1] + t * shade);
                    color[2] = (int) ((1 - t) * color[2] + t * shade);

                    raster.setPixel(j, i, color);
                } // for
            } // for
            g2D.drawImage(image, scaleImage, this);

        } // try
        catch (IOException e) {
            logger.log(Level.SEVERE, null, e);
        } // catch( IOException)
    } // imageProcessing( Graphics2D )

    /**
     * Modify this method to produce an image
     * of the Mandelbrot set.
     * 
     * You might want to begin by experimenting with
     * various patterns.
     * 
     * @param real
     * @param imag
     * @return 
     */
    private Color pixelColor(double real, double imag) {
        // Color all pixels that lie above the diagonal
        // line that runs from the upper left corner of
        // the image to the lower right corner one color,
        // and color the pixels above a different color.
        if (real < imag) {
            return Color.BLACK;
        } // if
        else {
            return Color.YELLOW;
        } // else

        // Paint each quadrant (upper left, upper right,
        // lower right, and lower left) of the image a
        // different color.
//        if( real < 0 && imag < 0 ) {
//            return Color.GREEN;
//        } // if
//        else if( real >= 0 && imag < 0 ) {
//            return Color.MAGENTA;
//        } // else if
//        else if( real >= 0 && imag >= 0) {
//            return Color.PINK;
//        } // else if
//        else {
//            return Color.CYAN;
//        }
        
        // Paint all pixels that lie within a circle
        // that is centered in the image one color,
        // and all other pixels another color.
//        if (real * real + imag * imag < 1.0) {
//            return Color.RED;
//        } // if
//        else {
//            return Color.WHITE;
//        } // else 
    } // pixelColor( double, double )
} // MandelBrotPanel
